__author__ = 'Helium'

# Here, we are importing the auxiliary functions to clean or convert data
from MarketPlaces.Utilities.utilities import *

# Here, we are importing BeautifulSoup to search through the HTML tree
from bs4 import BeautifulSoup


# This is the method to parse the Description Pages (one page to each Product in the Listing Pages)
def kingdom_description_parser(soup):

    # Fields to be parsed

    vendor = "-1"                       # 0 *Vendor_Name
    success = "-1"                      # 1 Vendor_Successful_Transactions
    rating_vendor = "-1"                # 2 Vendor_Rating
    name = "-1"                         # 3 *Product_Name
    describe = "-1"                     # 4 Product_Description
    CVE = "-1"                          # 5 Product_CVE_Classification (Common Vulnerabilities and Exposures)
    MS = "-1"                           # 6 Product_MS_Classification (Microsoft Security)
    category = "-1"                     # 7 Product_Category
    views = "-1"                        # 8 Product_Number_Of_Views
    reviews = "-1"                      # 9 Product_Number_Of_Reviews
    rating_item = "-1"                  # 10 Product_Rating
    addDate = "-1"                      # 11 Product_AddedDate
    BTC = "-1"                          # 12 Product_BTC_SellingPrice
    USD = "-1"                          # 13 Product_USD_SellingPrice
    EURO = "-1"                         # 14 Product_EURO_SellingPrice
    sold = "-1"                         # 15 Product_QuantitySold
    left = "-1"                         # 16 Product_QuantityLeft
    shipFrom = "-1"                     # 17 Product_ShippedFrom
    shipTo = "-1"                       # 18 Product_ShippedTo
    image = "-1"  # 19 Product_Image
    vendor_image = "-1"  # 20 Vendor_Image

    # Finding Product Name

    tag = soup.find('div', {"class": "col-md-9"})

    desc = tag.find('div',{"class": "col-md-8"}).find('div', {"class": "box-cont"})
    name = tag.find('div',{"class": "col-md-8"}).find('div', {"class": "box-head"}).text
    name = cleanString(name).strip()

    # Finding Prices
    # Kingdom prices can be shown in a variety of currencies, not all in USD, so keeping currency
    rows = desc.find_all('div', {"class", "row"}, recursive=False)
    USD = rows[-1].find('div', {"class": "row"}).find('h3').text
    USD = cleanNumbers(USD).strip()
    BTC = rows[-1].find('div', {"class": "row"}).find_next_sibling('div').find('span').text
    BTC = cleanNumbers(BTC).strip()

    # Finding Vendor
    vendor = rows[0].select_one('a[href^="/user"]').text
    vendor = cleanString(vendor).strip()

    # Finding Shipment Information (Origem)
    descs = rows[0].find_all('div', {"class": "col-md-3 text-right"})
    shipFrom = descs[2].text
    shipFrom = cleanString(shipFrom).strip()

    # Finding Shipment Information (Destiny)
    shipTo = rows[-1].find('div', {"class": "col-md-6"}).text
    shipTo = shipTo.replace("Ship to:","")
    shipTo = cleanString(shipTo).strip()
    if shipTo == '':
        shipTo = "-1"

    # Finding the Product Category
    category = descs[0].text
    category = cleanString(category).strip()

    # Finding the Product Quantity Available
    left = descs[1].text
    left = cleanString(left).strip()

    # Finding when the Product was Added
    dt = descs[-1].text.strip()
    addDate = datetime.strptime(dt, '%d.%m.%Y')

    # Finding the Product description
    describe = cleanString(soup.find('div', {"id": "descriptionContent"}).text).strip()

    # Finding the Number of Product Reviews
    reviews = str(len(soup.find('div', {"id": "feedbackContent"}).find_all(recursive=False)))

    # Searching for CVE and MS categories
    # no cve or ms in Kingdom

    # Populating the final variable (this should be a list with all fields scraped)

    row = (vendor, rating_vendor, success, name, describe, CVE, MS, category, views, reviews, rating_item, addDate,
           BTC, USD, EURO, sold, left, shipFrom, shipTo, image, vendor_image)

    # Sending the results

    return row


def kingdom_listing_parser(soup):

    # Fields to be parsed
    nm = 0                                    # *Total_Products (Should be Integer)
    mktName = "Kingdom"                      # 0 *Marketplace_Name
    vendor = []                               # 1 *Vendor y
    rating_vendor = []                        # 2 Vendor_Rating
    success = []                              # 3 Vendor_Successful_Transactions
    name = []                                 # 4 *Product_Name y
    CVE = []                                  # 5 Product_CVE_Classification (Common Vulnerabilities and Exposures)
    MS = []                                   # 6 Product_MS_Classification (Microsoft Security)
    category = []                             # 7 Product_Category y
    describe = []                             # 8 Product_Description
    views = []                                # 9 Product_Number_Of_Views
    reviews = []                              # 10 Product_Number_Of_Reviews
    rating_item = []                          # 11 Product_Rating
    addDate = []                              # 12 Product_AddDate
    BTC = []                                  # 13 Product_BTC_SellingPrice
    USD = []                                  # 14 Product_USD_SellingPrice y
    EURO = []                                 # 15 Product_EURO_SellingPrice
    sold = []                                 # 16 Product_QuantitySold
    qLeft =[]                                 # 17 Product_QuantityLeft
    shipFrom = []                             # 18 Product_ShippedFrom
    shipTo = []                               # 19 Product_ShippedTo
    image = []  # 20 Product_Image
    image_vendor = []  # 21 Vendor_Image
    href = []  # 22 Product_Links

    listing = soup.find('div', {"id": "p0"}).find('div').find_all('div', {"class": "row"}, recursive=False)

    # Populating the Number of Products
    nm = len(listing)

    for a in listing:

        # Finding Prices
        #in array USD, there may be prices not in USD, so includes currency as well
        prices = a.find('div', {"class": "col-md-3"})
        u = prices.find('h3').text
        USD.append(cleanNumbers(u).strip())
        bc = prices.find('div').find('span').text
        BTC.append(cleanNumbers(bc).strip())

        # Finding the Product
        product = a.find('div', {"class": "col-md-7"}).select_one('a[href^="/offer/view?"]').text
        name.append(cleanString(product).strip())

        # Finding Product Image
        product_image = a.find('img')
        product_image = product_image.get('src')
        product_image = product_image.split('base64,')[-1]
        image.append(product_image)

        # Finding the Vendor
        vendor_name = a.select_one('a[href^="/user"]').text
        vendor_name = vendor_name.replace('/', '')
        vendor.append(cleanString(vendor_name).strip())

        # Finding Views
        product_views = a.find('div', {"class": "col-md-7"}).find_all('p')[0].text
        views.append(cleanNumbers(product_views).strip())

        # Finding Sold
        product_sold = a.find('div', {"class": "base-label label label-rounded label-success"})
        if product_sold is not None:
            sold.append(cleanNumbers(product_sold.text).strip())
        else:
            sold.append("-1")

        # Adding the url to the list of urls
        link = a.find('div', {"class": "col-md-7"}).select_one('a[href^="/offer/view?"]')['href']
        href.append(link)

        # Searching for CVE and MS categories
        # cve and ms not in kingdom

    # Populate the final variable (this should be a list with all fields scraped)
    return organizeProducts(mktName, nm, vendor, rating_vendor, success, name, CVE, MS, category, describe, views,
                            reviews, rating_item, addDate, BTC, USD, EURO, sold, qLeft, shipFrom, shipTo, href,
                            image, image_vendor)


def kingdom_links_parser(soup):

    # Returning all links that should be visited by the Crawler

    href = []

    listing = soup.findAll('div', {"class": "col-md-7"})

    for a in listing:
        link = a.select_one('a[href^="/offer/view?"]')
        link = link['href']
        href.append(link)

    return href